@@ -40,21 +40,23 @@ if (!defined('STATUSNET') && !defined('LACONICA')) {
  * @license  http://www.fsf.org/licensing/licenses/agpl-3.0.html GNU Affero General Public License version 3.0
  * @link     http://status.net/
  */
-
 class RegisterAction extends Action
 {
     /**
      * Has there been an error?
      */
-
     var $error = null;
 
     /**
      * Have we registered?
      */
-
     var $registered = false;
 
+    /**
+     * Are we processing an invite?
+     */
+    var $invite = null;
+
     /**
      * Prepare page to run
      *
@@ -62,7 +64,6 @@ class RegisterAction extends Action
      * @param $args
      * @return string title
      */
-
     function prepare($args)
     {
         parent::prepare($args);
@@ -76,6 +77,7 @@ class RegisterAction extends Action
         }
 
         if (common_config('site', 'inviteonly') && empty($this->code)) {
+            // TRANS: Client error displayed when trying to register to an invite-only site without an invitation.
             $this->clientError(_('Sorry, only invited people can register.'));
             return false;
         }
@@ -83,6 +85,7 @@ class RegisterAction extends Action
         if (!empty($this->code)) {
             $this->invite = Invitation::staticGet('code', $this->code);
             if (empty($this->invite)) {
+            // TRANS: Client error displayed when trying to register to an invite-only site without a valid invitation.
                 $this->clientError(_('Sorry, invalid invitation code.'));
                 return false;
             }
@@ -99,13 +102,14 @@ class RegisterAction extends Action
      *
      * @return string title
      */
-
     function title()
     {
         if ($this->registered) {
+            // TRANS: Title for registration page after a succesful registration.
             return _('Registration successful');
         } else {
-            return _('Register');
+            // TRANS: Title for registration page.
+            return _m('TITLE','Register');
         }
     }
 
@@ -116,22 +120,19 @@ class RegisterAction extends Action
      *
      * Checks if registration is closed and shows an error if so.
      *
-     * Checks if only OpenID is allowed and redirects to openidlogin if so.
-     *
      * @param array $args $_REQUEST data
      *
      * @return void
      */
-
     function handle($args)
     {
         parent::handle($args);
 
         if (common_config('site', 'closed')) {
+            // TRANS: Client error displayed when trying to register to a closed site.
             $this->clientError(_('Registration not allowed.'));
-        } else if (common_config('site', 'openidonly')) {
-            common_redirect(common_local_url('openidlogin'));
         } else if (common_logged_in()) {
+            // TRANS: Client error displayed when trying to register while already logged in.
             $this->clientError(_('Already logged in.'));
         } else if ($_SERVER['REQUEST_METHOD'] == 'POST') {
             $this->tryRegister();
@@ -154,12 +155,12 @@ class RegisterAction extends Action
      *
      * @return void
      */
-
     function tryRegister()
     {
         if (Event::handle('StartRegistrationTry', array($this))) {
             $token = $this->trimmed('token');
             if (!$token || $token != common_session_token()) {
+                // TRANS: Client error displayed when the session token does not match or is not given.
                 $this->showForm(_('There was a problem with your session token. '.
                                   'Try again, please.'));
                 return;
@@ -184,50 +185,65 @@ class RegisterAction extends Action
             }
 
             if (common_config('site', 'inviteonly') && !($code && $invite)) {
+                // TRANS: Client error displayed when trying to register to an invite-only site without an invitation.
                 $this->clientError(_('Sorry, only invited people can register.'));
                 return;
             }
 
             // Input scrubbing
-            $nickname = common_canonical_nickname($nickname);
+            try {
+                $nickname = Nickname::normalize($nickname);
+            } catch (NicknameException $e) {
+                $this->showForm($e->getMessage());
+            }
             $email    = common_canonical_email($email);
 
             if (!$this->boolean('license')) {
-                $this->showForm(_('You can\'t register if you don\'t '.
+                // TRANS: Form validation error displayed when trying to register without agreeing to the site license.
+                $this->showForm(_('You cannot register if you do not '.
                                   'agree to the license.'));
-            } else if ($email && !Validate::email($email, true)) {
+            } else if ($email && !Validate::email($email, common_config('email', 'check_domain'))) {
+                // TRANS: Form validation error displayed when trying to register without a valid e-mail address.
                 $this->showForm(_('Not a valid email address.'));
-            } else if (!Validate::string($nickname, array('min_length' => 1,
-                                                          'max_length' => 64,
-                                                          'format' => NICKNAME_FMT))) {
-                $this->showForm(_('Nickname must have only lowercase letters '.
-                                  'and numbers and no spaces.'));
             } else if ($this->nicknameExists($nickname)) {
+                // TRANS: Form validation error displayed when trying to register with an existing nickname.
                 $this->showForm(_('Nickname already in use. Try another one.'));
             } else if (!User::allowed_nickname($nickname)) {
+                // TRANS: Form validation error displayed when trying to register with an invalid nickname.
                 $this->showForm(_('Not a valid nickname.'));
             } else if ($this->emailExists($email)) {
+                // TRANS: Form validation error displayed when trying to register with an already registered e-mail address.
                 $this->showForm(_('Email address already exists.'));
             } else if (!is_null($homepage) && (strlen($homepage) > 0) &&
                        !Validate::uri($homepage,
                                       array('allowed_schemes' =>
                                             array('http', 'https')))) {
+                // TRANS: Form validation error displayed when trying to register with an invalid homepage URL.
                 $this->showForm(_('Homepage is not a valid URL.'));
                 return;
             } else if (!is_null($fullname) && mb_strlen($fullname) > 255) {
-                $this->showForm(_('Full name is too long (max 255 chars).'));
+                // TRANS: Form validation error displayed when trying to register with a too long full name.
+                $this->showForm(_('Full name is too long (maximum 255 characters).'));
                 return;
-            } else if (!is_null($bio) && mb_strlen($bio) > 140) {
-                $this->showForm(_('Bio is too long (max 140 chars).'));
+            } else if (Profile::bioTooLong($bio)) {
+                // TRANS: Form validation error on registration page when providing too long a bio text.
+                // TRANS: %d is the maximum number of characters for bio; used for plural.
+                $this->showForm(sprintf(_m('Bio is too long (maximum %d character).',
+                                           'Bio is too long (maximum %d characters).',
+                                           Profile::maxBio()),
+                                        Profile::maxBio()));
                 return;
             } else if (!is_null($location) && mb_strlen($location) > 255) {
-                $this->showForm(_('Location is too long (max 255 chars).'));
+                // TRANS: Form validation error displayed when trying to register with a too long location.
+                $this->showForm(_('Location is too long (maximum 255 characters).'));
                 return;
             } else if (strlen($password) < 6) {
+                // TRANS: Form validation error displayed when trying to register with too short a password.
                 $this->showForm(_('Password must be 6 or more characters.'));
                 return;
             } else if ($password != $confirm) {
-                $this->showForm(_('Passwords don\'t match.'));
+                // TRANS: Form validation error displayed when trying to register with non-matching passwords.
+                $this->showForm(_('Passwords do not match.'));
             } else if ($user = User::register(array('nickname' => $nickname,
                                                     'password' => $password,
                                                     'email' => $email,
@@ -237,11 +253,13 @@ class RegisterAction extends Action
                                                     'location' => $location,
                                                     'code' => $code))) {
                 if (!$user) {
+                    // TRANS: Form validation error displayed when trying to register with an invalid username or password.
                     $this->showForm(_('Invalid username or password.'));
                     return;
                 }
                 // success!
                 if (!common_set_user($user)) {
+                    // TRANS: Server error displayed when saving fails during user registration.
                     $this->serverError(_('Error setting user.'));
                     return;
                 }
@@ -252,12 +270,14 @@ class RegisterAction extends Action
                     common_rememberme($user);
                 }
 
-                Event::handle('EndRegistrationTry', array($this));
-
                 // Re-init language env in case it changed (not yet, but soon)
                 common_init_language();
+
+                Event::handle('EndRegistrationTry', array($this));
+
                 $this->showSuccess();
             } else {
+                // TRANS: Form validation error displayed when trying to register with an invalid username or password.
                 $this->showForm(_('Invalid username or password.'));
             }
         }
@@ -272,11 +292,10 @@ class RegisterAction extends Action
      *
      * @return boolean true if the nickname already exists
      */
-
     function nicknameExists($nickname)
     {
         $user = User::staticGet('nickname', $nickname);
-        return ($user !== false);
+        return is_object($user);
     }
 
     /**
@@ -288,7 +307,6 @@ class RegisterAction extends Action
      *
      * @return boolean true if the address already exists
      */
-
     function emailExists($email)
     {
         $email = common_canonical_email($email);
@@ -296,7 +314,7 @@ class RegisterAction extends Action
             return false;
         }
         $user = User::staticGet('email', $email);
-        return ($user !== false);
+        return is_object($user);
     }
 
     // overrrided to add entry-title class
@@ -327,7 +345,6 @@ class RegisterAction extends Action
      *
      * @return void
      */
-
     function showPageNotice()
     {
         if ($this->registered) {
@@ -335,22 +352,12 @@ class RegisterAction extends Action
         } else if ($this->error) {
             $this->element('p', 'error', $this->error);
         } else {
-            if (common_config('openid', 'enabled')) {
-                $instr =
-                  common_markup_to_html(_('With this form you can create '.
-                                          ' a new account. ' .
-                                          'You can then post notices and '.
-                                          'link up to friends and colleagues. '.
-                                          '(Have an [OpenID](http://openid.net/)? ' .
-                                          'Try our [OpenID registration]'.
-                                          '(%%action.openidlogin%%)!)'));
-            } else {
-                $instr =
-                  common_markup_to_html(_('With this form you can create '.
-                                          ' a new account. ' .
-                                          'You can then post notices and '.
-                                          'link up to friends and colleagues.'));
-            }
+            $instr =
+              // TRANS: Page notice on registration page.
+              common_markup_to_html(_('With this form you can create '.
+                                      'a new account. ' .
+                                      'You can then post notices and '.
+                                      'link up to friends and colleagues.'));
 
             $this->elementStart('div', 'instructions');
             $this->raw($instr);
@@ -367,7 +374,6 @@ class RegisterAction extends Action
      *
      * @return void
      */
-
     function showForm($error=null)
     {
         $this->error = $error;
@@ -382,7 +388,6 @@ class RegisterAction extends Action
      *
      * @return void
      */
-
     function showContent()
     {
         if ($this->registered) {
@@ -397,7 +402,6 @@ class RegisterAction extends Action
      *
      * @return void
      */
-
     function showFormContent()
     {
         $code = $this->trimmed('code');
@@ -409,6 +413,7 @@ class RegisterAction extends Action
         }
 
         if (common_config('site', 'inviteonly') && !($code && $invite)) {
+            // TRANS: Client error displayed when trying to register to an invite-only site without an invitation.
             $this->clientError(_('Sorry, only invited people can register.'));
             return;
         }
@@ -418,6 +423,7 @@ class RegisterAction extends Action
                                           'class' => 'form_settings',
                                           'action' => common_local_url('register')));
         $this->elementStart('fieldset');
+        // TRANS: Fieldset legend on accout registration page.
         $this->element('legend', null, 'Account settings');
         $this->hidden('token', common_session_token());
 
@@ -428,56 +434,86 @@ class RegisterAction extends Action
         $this->elementStart('ul', 'form_data');
         if (Event::handle('StartRegistrationFormData', array($this))) {
             $this->elementStart('li');
+            // TRANS: Field label on account registration page.
             $this->input('nickname', _('Nickname'), $this->trimmed('nickname'),
-                         _('1-64 lowercase letters or numbers, '.
-                           'no punctuation or spaces. Required.'));
+                         // TRANS: Field title on account registration page.
+                         _('1-64 lowercase letters or numbers, no punctuation or spaces.'));
             $this->elementEnd('li');
             $this->elementStart('li');
+            // TRANS: Field label on account registration page.
             $this->password('password', _('Password'),
-                            _('6 or more characters. Required.'));
+                            // TRANS: Field title on account registration page.
+                            _('6 or more characters.'));
             $this->elementEnd('li');
             $this->elementStart('li');
-            $this->password('confirm', _('Confirm'),
-                            _('Same as password above. Required.'));
+            // TRANS: Field label on account registration page. In this field the password has to be entered a second time.
+            $this->password('confirm', _m('PASSWORD','Confirm'),
+                         // TRANS: Field title on account registration page.
+                         _('Same as password above.'));
             $this->elementEnd('li');
             $this->elementStart('li');
             if ($this->invite && $this->invite->address_type == 'email') {
-                $this->input('email', _('Email'), $this->invite->address,
+                // TRANS: Field label on account registration page.
+                $this->input('email', _m('LABEL','Email'), $this->invite->address,
+                             // TRANS: Field title on account registration page.
                              _('Used only for updates, announcements, '.
-                               'and password recovery'));
+                               'and password recovery.'));
             } else {
-                $this->input('email', _('Email'), $this->trimmed('email'),
+                // TRANS: Field label on account registration page.
+                $this->input('email', _m('LABEL','Email'), $this->trimmed('email'),
+                             // TRANS: Field title on account registration page.
                              _('Used only for updates, announcements, '.
-                               'and password recovery'));
+                               'and password recovery.'));
             }
             $this->elementEnd('li');
             $this->elementStart('li');
+            // TRANS: Field label on account registration page.
             $this->input('fullname', _('Full name'),
                          $this->trimmed('fullname'),
-                         _('Longer name, preferably your "real" name'));
+                     // TRANS: Field title on account registration page.
+                     _('Longer name, preferably your "real" name.'));
             $this->elementEnd('li');
             $this->elementStart('li');
+            // TRANS: Field label on account registration page.
             $this->input('homepage', _('Homepage'),
                          $this->trimmed('homepage'),
+                         // TRANS: Field title on account registration page.
                          _('URL of your homepage, blog, '.
-                           'or profile on another site'));
+                           'or profile on another site.'));
             $this->elementEnd('li');
             $this->elementStart('li');
+            $maxBio = Profile::maxBio();
+            if ($maxBio > 0) {
+                // TRANS: Text area title in form for account registration. Plural
+                // TRANS: is decided by the number of characters available for the
+                // TRANS: biography (%d).
+                $bioInstr = sprintf(_m('Describe yourself and your interests in %d character.',
+                                       'Describe yourself and your interests in %d characters.',
+                                       $maxBio),
+                                    $maxBio);
+            } else {
+                // TRANS: Text area title on account registration page.
+                $bioInstr = _('Describe yourself and your interests.');
+            }
+            // TRANS: Text area label on account registration page.
             $this->textarea('bio', _('Bio'),
                             $this->trimmed('bio'),
-                            _('Describe yourself and your '.
-                              'interests in 140 chars'));
+                            $bioInstr);
             $this->elementEnd('li');
             $this->elementStart('li');
+            // TRANS: Field label on account registration page.
             $this->input('location', _('Location'),
                          $this->trimmed('location'),
+                         // TRANS: Field title on account registration page.
                          _('Where you are, like "City, '.
-                           'State (or Region), Country"'));
+                           'State (or Region), Country".'));
             $this->elementEnd('li');
             Event::handle('EndRegistrationFormData', array($this));
             $this->elementStart('li', array('id' => 'settings_rememberme'));
+            // TRANS: Checkbox label on account registration page.
             $this->checkbox('rememberme', _('Remember me'),
                             $this->boolean('rememberme'),
+                            // TRANS: Checkbox title on account registration page.
                             _('Automatically login in the future; '.
                               'not for shared computers!'));
             $this->elementEnd('li');
@@ -492,20 +528,61 @@ class RegisterAction extends Action
             $this->elementStart('li');
             $this->element('input', $attrs);
             $this->elementStart('label', array('class' => 'checkbox', 'for' => 'license'));
-            $this->text(_('My text and files are available under '));
-            $this->element('a', array('href' => common_config('license', 'url')),
-                           common_config('license', 'title'), _("Creative Commons Attribution 3.0"));
-            $this->text(_(' except this private data: password, '.
-                          'email address, IM address, and phone number.'));
+            $this->raw($this->licenseCheckbox());
             $this->elementEnd('label');
             $this->elementEnd('li');
         }
         $this->elementEnd('ul');
-        $this->submit('submit', _('Register'));
+        // TRANS: Button text to register a user on account registration page.
+        $this->submit('submit', _m('BUTTON','Register'));
         $this->elementEnd('fieldset');
         $this->elementEnd('form');
     }
 
+    function licenseCheckbox()
+    {
+        $out = '';
+        switch (common_config('license', 'type')) {
+        case 'private':
+            $out .= htmlspecialchars(sprintf(
+                // TRANS: Copyright checkbox label in registration dialog, for private sites.
+                // TRANS: %1$s is the StatusNet sitename.
+                _('I understand that content and data of %1$s are private and confidential.'),
+                common_config('site', 'name')));
+            // fall through
+        case 'allrightsreserved':
+            if ($out != '') {
+                $out .= ' ';
+            }
+            if (common_config('license', 'owner')) {
+                $out .= htmlspecialchars(sprintf(
+                    // TRANS: Copyright checkbox label in registration dialog, for all rights reserved with a specified copyright owner.
+                    // TRANS: %1$s is the license owner.
+                    _('My text and files are copyright by %1$s.'),
+                    common_config('license', 'owner')));
+            } else {
+                // TRANS: Copyright checkbox label in registration dialog, for all rights reserved with ownership left to contributors.
+                $out .= htmlspecialchars(_('My text and files remain under my own copyright.'));
+            }
+            // TRANS: Copyright checkbox label in registration dialog, for all rights reserved.
+            $out .= ' ' . _('All rights reserved.');
+            break;
+        case 'cc': // fall through
+        default:
+            // TRANS: Copyright checkbox label in registration dialog, for Creative Commons-style licenses.
+            $message = _('My text and files are available under %s ' .
+                         'except this private data: password, ' .
+                         'email address, IM address, and phone number.');
+            $link = '<a href="' .
+                    htmlspecialchars(common_config('license', 'url')) .
+                    '">' .
+                    htmlspecialchars(common_config('license', 'title')) .
+                    '</a>';
+            $out .= sprintf(htmlspecialchars($message), $link);
+        }
+        return $out;
+    }
+
     /**
      * Show some information about registering for the site
      *
@@ -513,7 +590,6 @@ class RegisterAction extends Action
      *
      * @return void
      */
-
     function showSuccess()
     {
         $this->registered = true;
@@ -527,46 +603,54 @@ class RegisterAction extends Action
      *
      * @return void
      */
-
     function showSuccessContent()
     {
-        $nickname = $this->arg('nickname');
+        if (Event::handle('StartRegisterSuccess', array($this))) {
+            $nickname = $this->arg('nickname');
 
-        $profileurl = common_local_url('showstream',
-                                       array('nickname' => $nickname));
+            $profileurl = common_local_url('showstream',
+                                           array('nickname' => $nickname));
 
-        $this->elementStart('div', 'success');
-        $instr = sprintf(_('Congratulations, %s! And welcome to %%%%site.name%%%%. '.
-                           'From here, you may want to...'. "\n\n" .
-                           '* Go to [your profile](%s) '.
-                           'and post your first message.' .  "\n" .
-                           '* Add a [Jabber/GTalk address]'.
-                           '(%%%%action.imsettings%%%%) '.
-                           'so you can send notices '.
-                           'through instant messages.' . "\n" .
-                           '* [Search for people](%%%%action.peoplesearch%%%%) '.
-                           'that you may know or '.
-                           'that share your interests. ' . "\n" .
-                           '* Update your [profile settings]'.
-                           '(%%%%action.profilesettings%%%%)'.
-                           ' to tell others more about you. ' . "\n" .
-                           '* Read over the [online docs](%%%%doc.help%%%%)'.
-                           ' for features you may have missed. ' . "\n\n" .
-                           'Thanks for signing up and we hope '.
-                           'you enjoy using this service.'),
-                         $nickname, $profileurl);
+            $this->elementStart('div', 'success');
+            // TRANS: Text displayed after successful account registration.
+            // TRANS: %1$s is the registered nickname, %2$s is the profile URL.
+            // TRANS: This message contains Markdown links in the form [link text](link)
+            // TRANS: and variables in the form %%%%variable%%%%. Please mind the syntax.
+            $instr = sprintf(_('Congratulations, %1$s! And welcome to %%%%site.name%%%%. '.
+                               'From here, you may want to...'. "\n\n" .
+                               '* Go to [your profile](%2$s) '.
+                               'and post your first message.' .  "\n" .
+                               '* Add a [Jabber/GTalk address]'.
+                               '(%%%%action.imsettings%%%%) '.
+                               'so you can send notices '.
+                               'through instant messages.' . "\n" .
+                               '* [Search for people](%%%%action.peoplesearch%%%%) '.
+                               'that you may know or '.
+                               'that share your interests. ' . "\n" .
+                               '* Update your [profile settings]'.
+                               '(%%%%action.profilesettings%%%%)'.
+                               ' to tell others more about you. ' . "\n" .
+                               '* Read over the [online docs](%%%%doc.help%%%%)'.
+                               ' for features you may have missed. ' . "\n\n" .
+                               'Thanks for signing up and we hope '.
+                               'you enjoy using this service.'),
+                             $nickname, $profileurl);
 
-        $this->raw(common_markup_to_html($instr));
+            $this->raw(common_markup_to_html($instr));
 
-        $have_email = $this->trimmed('email');
-        if ($have_email) {
-            $emailinstr = _('(You should receive a message by email '.
-                            'momentarily, with ' .
-                            'instructions on how to confirm '.
-                            'your email address.)');
-            $this->raw(common_markup_to_html($emailinstr));
+            $have_email = $this->trimmed('email');
+            if ($have_email) {
+                // TRANS: Instruction text on how to deal with the e-mail address confirmation e-mail.
+                $emailinstr = _('(You should receive a message by email '.
+                                'momentarily, with ' .
+                                'instructions on how to confirm '.
+                                'your email address.)');
+                $this->raw(common_markup_to_html($emailinstr));
+            }
+            $this->elementEnd('div');
+
+            Event::handle('EndRegisterSuccess', array($this));
         }
-        $this->elementEnd('div');
     }
 
     /**
@@ -574,11 +658,25 @@ class RegisterAction extends Action
      *
      * @return void
      */
-
     function showLocalNav()
     {
-        $nav = new LoginGroupNav($this);
-        $nav->show();
+        if (common_logged_in()) {
+            parent::showLocalNav();
+        } else {
+            $nav = new LoginGroupNav($this);
+            $nav->show();
+        }
+    }
+
+    /**
+     * Show a bit of login context
+     *
+     * @return nothing
+     */
+    function showProfileBlock()
+    {
+        if (common_logged_in()) {
+            parent::showProfileBlock();
+        }
     }
 }
-
