@@ -1,7 +1,7 @@
 jQuery(document).ready(function($){
   $('notices_primary').infinitescroll({
     debug: false,
-    infiniteScroll  : false,
+    infiniteScroll  : !infinite_scroll_on_next_only,
     nextSelector    : 'body#public li.nav_next a,'+
                       'body#all li.nav_next a,'+
                       'body#showstream li.nav_next a,'+
@@ -12,12 +12,16 @@ jQuery(document).ready(function($){
     loadingImg      : $('address .url')[0].href+'plugins/InfiniteScroll/ajax-loader.gif',
     text            : "<em>Loading the next set of posts...</em>",
     donetext        : "<em>Congratulations, you\'ve reached the end of the Internet.</em>",
-    navSelector     : "div.pagination",
+    navSelector     : "#pagination",
     contentSelector : "#notices_primary ol.notices",
-    itemSelector    : "#notices_primary ol.notices li"
+    itemSelector    : "#notices_primary ol.notices > li"
     },function(){
-        NoticeAttachments();
-        NoticeReply();
-        NoticeFavors();
+        // Reply button and attachment magic need to be set up
+        // for each new notice.
+        // DO NOT run SN.Init.Notices() which will duplicate stuff.
+        $(this).find('.notice').each(function() {
+            SN.U.NoticeReplyTo($(this));
+            SN.U.NoticeWithAttachment($(this));
+        });
     });
 });
